// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: ApproxPhotonDistribution
// Author: Hassan Siddiqui
//
// Description: 
// Models incident photon distribution on detector.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: ApproxPhotonDistribution.h,v 1.2 1999/02/17 13:29:38 hsiddiqu Exp $

#ifndef ApproxPhotonDistribution_H
#define ApproxPhotonDistribution_H

// General classes

#include <string>
#include <iostream>
#include <vector>
#include <cmath>
#include<cassert>

// SAS-specific classes

#include "Dal.h"

// Utility functions
#include "Tracer.h"

// Local functions
#include "PhotonDistribution.h"
#include "CountsSpectrum.h"
#include "Interpolator.h"

// ========================================================================

// class ApproxPhotonDistribution

// Models incident photon distribution on detector, using a CountsSpectrum
// object as an approximation to (a subset of) the distribution. 

// ========================================================================

class ApproxPhotonDistribution : public PhotonDistribution
{

public:

  /* Constructor */

  ApproxPhotonDistribution( const CountsSpectrum  * countsSpectrum,
			    double                  energyLow,
			    double                  energyHigh):
    _countsSpectrum (countsSpectrum),
    _energyLow      (energyLow),
    _energyHigh     (energyHigh)
    {
      PROC;
 
      // Set up members used to interpolate countspectrum. 
      // for determining input flux model as fn of energy
      _firstChannel         = _countsSpectrum->firstChannel(); 
      _lastChannel          = _countsSpectrum->lastChannel(); 

      ENDPROC;
    }

  /* Determine flux (/s) for given energy */
  double flux(double energy ) const
    {
      PROC;

      //      cout << " ApproxPhotonDistribution: energy " << energy << endl;
      

      /* Get closest channel pair corresponding to energy */

      unsigned int channelLow;
      unsigned int channelHigh;

      double channel = Interpolator::calc(energy,
					  _energyLow, 
					  _energyHigh,
					  _firstChannel,
					  _lastChannel);
      if (channel > _lastChannel)
	{
	  /* Get last two channel values */
	  channelHigh = _lastChannel;
	  channelLow = channelHigh-1;
	  assert(channelLow >= _firstChannel);
	}
      else if (channel < _firstChannel)
	{
	  /* Get first two channel values */
	  channelLow  = _firstChannel;
	  channelHigh = channelLow + 1;
	  assert(channelHigh <= _lastChannel);
	}
      else
	{
	  /* Round down to nearest integer */
	  channelLow = (unsigned int) channel;
	  channelHigh = channelLow + 1;
	}

      /* Determine countrate. */
      double countrateLow =  _countsSpectrum->countrate( channelLow );
      double countrateHigh = _countsSpectrum->countrate( channelHigh );

      /* Interpolate countrate. */
      
      double countrate = 
	Interpolator::calc(channel,
			   channelLow, 
			   channelHigh,
			   countrateLow,
			   countrateHigh);
      

//       cout << " ApproxPhotonDistribution: "  
// 	   << " countrate for" << energy
// 	   << " (ie channel "  << channel 
// 	   << ")  is "         << countrate
// 	   << endl;


      RETURN countrate;

    }

private:

  // Suppress default constructor
  ApproxPhotonDistribution();

  const CountsSpectrum * _countsSpectrum;
  unsigned int           _firstChannel;
  unsigned int           _lastChannel;
  double                 _energyLow;
  double                 _energyHigh;

};
#endif




