// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: DummySplitEventsData
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: DummySplitEventsData.cc,v 1.1 2000/03/28 12:42:49 hsiddiqu Exp $

/* General */
#include<cassert>

/* SAS-specific */
#include "errstr.h"
#include "Dal.h"

/* Local */
#include "DummySplitEventsData.h"

// =================================================================

// class DummySplitEventsData

// This implementation loads in the split event distribution 
// from a Dal dataset.

// =================================================================

/*  Constructors */
DummySplitEventsData::DummySplitEventsData( unsigned int numberOfPatterns,
					    const string& splitEventsFile):
  SplitEventsData(numberOfPatterns),
  _data(numberOfPatterns)
{
  PROC;
  
  /* Open split events datafile */
  DataSet * set = dataSetServer->
    open( splitEventsFile, DataSet::Read);
  
  Table       * table              = set->table("PATTERNDIST");
  
  Column      * energyCol          = table->column("ENERGY");
  ColumnData  * energyColData      = energyCol->data();
  assert(energyColData->dataType() == Dal::Real32);
  real32      * energyData         = energyColData->real32Data();
      
  Column      * patternCol     = table->column("FRACTION");
  ColumnData  * patternColData = patternCol->data();
  
  /* Check size of column vector */
  assert (patternColData->elements() == numberOfPatterns);

  for (unsigned int row = 0; row < table->rows(); ++row)
    {
      
      /* Get split event distribution for row */
      SplitEventsDist splitEventsRow;
      
      for ( unsigned int p = 0; 
	    p < numberOfPatterns; 
	    ++p)
	{
	    assert ( (row * numberOfPatterns + p) 
		     <= (numberOfPatterns * numberOfPatterns) );
	    splitEventsRow.
	      push_back( (patternColData->real32Data())
			 [(row * numberOfPatterns) + p] );
	  }
	
	/* Add row to map */
	_data.add(energyData[row], splitEventsRow);
      }
        
      delete energyColData;

      if (!_data.check())
	{
	  Sas::error("tooFewDataPoints",
		"Not enough data points in dataset - need at least two.");
	}
    
      ENDPROC;
};

  /*  Destructors */
DummySplitEventsData::~DummySplitEventsData(){};

/* Get interpolated Pattern information for given energy */
const vector<double> DummySplitEventsData::dist( double energy ) const
{
  PROC;
  RETURN _data.value(energy);
};
