// 
// Module: MOSBiPixelPatternFuncs
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: MOSBiPixelPatternFuncs.cc,v 1.1 2000/03/28 12:42:50 hsiddiqu Exp $

/* General classes */
#include <cmath>

/* SAS-specific */
#include "Dal.h"
#include "Tracer.h"

/* Local classes */
#include "MOSBiPixelPatternFuncs.h"
#include "ProbabilityFunctions.h"


// ==================================================================

// class MOSBiPixelPatternFuncs

// Contains algorithms to correct for pile-up where the pattern of interest
// is of Bi-pixel type.

// ==================================================================

  /* Constructors */
MOSBiPixelPatternFuncs::MOSBiPixelPatternFuncs(const SplitEventsData & splitEventsData) :
  SplitPatternFuncs(splitEventsData){};

MOSBiPixelPatternFuncs::~MOSBiPixelPatternFuncs(){};

/* Note flux is in units of /frame/pixel */
const PileupData MOSBiPixelPatternFuncs::getPileup( const SplitEventsDist& alpha,
						    double flux )  const
{
      PROC;

      double gamma = 12 + (3.5 * alpha[1]) + (7 * alpha[2]) + (8 * alpha[3]);

      double p1 = ProbabilityFunctions::prob1(alpha, flux);
      double p2 = ProbabilityFunctions::prob2(alpha, flux);

      double mu = 2*(p2 + (1-p2)*(p1*p1)) * 
	exp(-(gamma -2*alpha[0] - 0.5*alpha[1])*flux);

      double mu_t  =  alpha[0] * flux * exp(-gamma*flux);


//       cout << " flux "     << flux     << endl
// 	   << " alpha[0] " << alpha[0] << endl
// 	   << " alpha[1] " << alpha[1] << endl
// 	   << " mu "       << mu       << endl
// 	   << " mu_t "     << mu_t     << endl;

      RETURN PileupData(mu, mu_t);
}
