// 
// Module: MOSMonoPixelFuncs
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: MOSMonoPixelPatternFuncs.cc,v 1.1 2000/03/28 12:42:51 hsiddiqu Exp $

/* General classes */

/* SAS-specific */
#include "Tracer.h"

/* Local classes */
#include "MOSMonoPixelPatternFuncs.h"


// =============================================================

// class MOSMonoPixelPatternFuncs

// Contains algorithms to correct for pile-up where the pattern of interest
// is of mono-pixel type.

// =============================================================

/* Constructors */
MOSMonoPixelPatternFuncs::MOSMonoPixelPatternFuncs(const SplitEventsData & splitEventsData) :
  SplitPatternFuncs(splitEventsData){};

MOSMonoPixelPatternFuncs::~MOSMonoPixelPatternFuncs(){};

const PileupData MOSMonoPixelPatternFuncs::getPileup( const SplitEventsDist& alpha,
						      double flux )  const
{
  PROC;
  
  double alpha_flux     = alpha[0] * flux;
  
  double gamma = 9 + (3 * alpha[1]) + (6 * alpha[2]) + (7 * alpha[3]);
  
  double exp_gamma_flux = exp(-gamma*flux);
  
  double mu             = ( exp(alpha_flux) - 1) * exp_gamma_flux;
  
  double mu_t           =  alpha_flux * exp_gamma_flux;


//       cout << " flux "    << flux    << endl
// 	   << " alpha[0] " << alpha[0] << endl
// 	   << " mu "      << mu      << endl
// 	   << " mu_t "    << mu_t    << endl;

  RETURN PileupData(mu, mu_t);
}
