// 
// Module: MOSQuadPixelPatternFuncs
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: MOSQuadPixelPatternFuncs.cc,v 1.1 2000/03/28 12:42:52 hsiddiqu Exp $

/* General classes */
#include <cmath>

/* SAS-specific */
#include "Dal.h"
#include "Tracer.h"

/* Local classes */
#include "MOSQuadPixelPatternFuncs.h"
#include "ProbabilityFunctions.h"


// ==================================================================

// class MOSQuadPixelPatternFuncs

// Contains algorithms to correct for pile-up where the pattern of interest
// is of Quad-pixel type.

// ==================================================================

/* Constructors */
MOSQuadPixelPatternFuncs::MOSQuadPixelPatternFuncs( const SplitEventsData & splitEventsData) :
  SplitPatternFuncs(splitEventsData){};

MOSQuadPixelPatternFuncs::~MOSQuadPixelPatternFuncs(){};

/* Note flux is in units of /frame/pixel */
const PileupData MOSQuadPixelPatternFuncs::getPileup( const SplitEventsDist& alpha,
			       double flux )  const
    {
      PROC;

      double gamma = 16 + (4 * alpha[1]) + (8 * alpha[2]) + (9 * alpha[3]);

      double p1 = ProbabilityFunctions::prob1(alpha, flux);
      double p2 = ProbabilityFunctions::prob2(alpha, flux);

      double p2sqd = p2*p2;

      double q2 = 
	( p1*p1*p1*p1 * exp(-2.  * alpha[1] * flux) ) +
	( 4*p1*p1*p2  * exp(-1.5 * alpha[2] * flux) ) +
	( 4*p1*p2sqd  * exp(-alpha[1]*flux) ) +
	( p2sqd*(1-p2sqd) );   

      double q3 =
	1 - ( exp(-alpha[2]*flux) * (1-q2)) -
	(4*(1-exp(-0.25*alpha[2]*flux))*
	exp(-(alpha[0] + alpha[1] + 0.75*alpha[2])*flux));
      
      double mu = 
	(1 - (exp(-alpha[3]*flux)*(1-q3))) *
	( exp(-(gamma - 1 - 3*alpha[0] - alpha[1])*flux)) ; 

      double mu_t  =  alpha[3] * flux * exp(-gamma*flux);


//       cout << " flux "     << flux     << endl
// 	   << " alpha[0] " << alpha[0] << endl
// 	   << " alpha[1] " << alpha[1] << endl
// 	   << " alpha[2] " << alpha[2] << endl
// 	   << " alpha[3] " << alpha[3] << endl
// 	   << " mu "       << mu       << endl
// 	   << " mu_t "     << mu_t     << endl;

      RETURN PileupData(mu, mu_t);
    }
