// 
// Module: MOSTriPixelPatternFuncs
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: MOSTriPixelPatternFuncs.cc,v 1.1 2000/03/28 12:42:52 hsiddiqu Exp $

/* General classes */
#include <cmath>

/* SAS-specific */
#include "Dal.h"
#include "Tracer.h"

/* Local classes */
#include "MOSTriPixelPatternFuncs.h"
#include "ProbabilityFunctions.h"


// ==================================================================

// class MOSTriPixelPatternFuncs

// Contains algorithms to correct for pile-up where the pattern of interest
// is of Tri-pixel type.

// ==================================================================

/* Constructors */
MOSTriPixelPatternFuncs::MOSTriPixelPatternFuncs(const SplitEventsData & splitEventsData) :
  SplitPatternFuncs(splitEventsData){};

MOSTriPixelPatternFuncs::~MOSTriPixelPatternFuncs(){};


/* Note flux is in units of /frame/pixel */
const PileupData MOSTriPixelPatternFuncs::getPileup( const SplitEventsDist& alpha,
						     double flux )  const
    {
      PROC;

      double gamma = 15 + (4 * alpha[1]) + (7.75 * alpha[2]) + (9 * alpha[3]);

      double p1 = ProbabilityFunctions::prob1(alpha, flux);
      double p2 = ProbabilityFunctions::prob2(alpha, flux);

      double t3 = 
	( p1*p1*p1/3. * exp(-alpha[1]*flux )) +
	( p1*p2/2.    * exp(-alpha[1]*flux/2. )) +
	( 0.75*p2*p2 );

      double mu = 
	4 * ( 1 - (exp(-0.25*alpha[2]*flux) * (1 - t3))) *
	( exp(-(gamma - 3*alpha[0] - alpha[1] - 0.25*alpha[2])*flux) ); 

      double mu_t  =  alpha[2] * flux * exp(-gamma*flux);


//       cout << " flux "     << flux     << endl
// 	   << " alpha[0] " << alpha[0] << endl
// 	   << " alpha[1] " << alpha[1] << endl
// 	   << " alpha[2] " << alpha[2] << endl
// 	   << " mu "       << mu       << endl
// 	   << " mu_t "     << mu_t     << endl;

      RETURN PileupData(mu, mu_t);
    }
