// Module: NonUnifBinnedAxis.h
// Author: Hassan Siddiqui
//
// Description:
//
// A BinnedAxis implementaton that uses an IntervalVector as input 
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: NonUnifBinnedAxis.cc,v 1.1 2000/03/28 12:42:53 hsiddiqu Exp $

/* General classes */

/* SAS-specific classes */
#include "errstr.h"
#include "Tracer.h"
#include "CalCommon.h"

/* Local classes */
#include "NonUnifBinnedAxis.h"

/***********************************************************
 *
 * class NonUnifBinnedAxis
 *
 ***********************************************************/

NonUnifBinnedAxis::NonUnifBinnedAxis 
(const IntervalVector<double> & in):
  BinnedAxis(0,1),
  _bins(0)
{
  PROC;
  _intervalVector.erase(_intervalVector.begin(), 
			_intervalVector.end());

  for (unsigned i = 0; i < in.size(); ++i)
    {
      _intervalVector.push_back(in[i]);
    }
  
  _bins  = in.size();
  _lower = in[0].lo();
  _upper = in[_bins-1].hi();
  ENDPROC;
}

/* This constructor is used to make a copy of another
   binned axis object. Useful to convert a reference
   to a dynamic object */
NonUnifBinnedAxis::NonUnifBinnedAxis 
(const BinnedAxis & in):
  BinnedAxis(0,1),
  _bins(0)
{
  PROC;
  _intervalVector.erase(_intervalVector.begin(), 
			_intervalVector.end());

  for (unsigned i = 0; i < in.bins(); ++i)
    {
      _intervalVector.push_back(in.bin(i));
    }
  
  _bins  = in.bins();
  _lower = in.bin(unsigned(0)).lo();
  _upper = in.bin(_bins-1).hi();
  ENDPROC;
}

NonUnifBinnedAxis::~NonUnifBinnedAxis(){}

unsigned NonUnifBinnedAxis::bins() const
{
  PROC;
  RETURN _bins;
}

Bin NonUnifBinnedAxis::bin(unsigned index) const
{
  PROC;
  
  if (index > _bins)
    {
      errstr << " NonUnifBinnedAxis: Index requested " << index
	     << " exceeds number of bins " << _bins
	     << Sas::error("InvalidIndexNonUnifBinnedAxis");
      ALTRETURNV(Bin(0,0));
    }

  RETURN _intervalVector[index];
}

Bin NonUnifBinnedAxis::bin(double value) const
{
  RETURN _intervalVector[index(value)];
}

unsigned NonUnifBinnedAxis::index(double value) const
{
  PROC;

  if (value < _lower || value >= _upper)
    {
      errstr << " NonUnifBinnedAxis: value requested " << value
	     << " exceeds range (" << _lower << "," <<_upper << ")"
	     << Sas::error("InvalidValueNonUnifBinnedAxis");

      ALTRETURNV(0);
    }

  RETURN _intervalVector.whichInterval(value);
}


/* Dummy implementations */
unsigned & NonUnifBinnedAxis::operator[](unsigned /* index */)
{
  PROC;
  RETURN _bins;
}

unsigned & NonUnifBinnedAxis::operator[](double /* value */)
{ 
  PROC;
  RETURN _bins;
}



