// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: PatternFuncs
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: PatternFuncs.h,v 1.2 1999/08/20 10:37:15 hsiddiqu Exp $

#ifndef PatternFuncs_H
#define PatternFuncs_H

// SAS-specific classes
#include "Dal.h"

// General classes
#include <cmath>

// General utility functions
#include "Tracer.h"

// Local classes

// ===========================================================================

// class PileupData

// Contains the computed true and piled-up countrate pair.

// Also computes the flux loss and pile-up fraction.

// ===========================================================================
class PileupData
{

public:

  /* Constructor */
  PileupData( double piledupCtRate,
	      double trueCtRate,
	      double probSingleEvent):
    _piledupCtRate(piledupCtRate),
    _trueCtRate(trueCtRate),
    _probSingleEvent(probSingleEvent)
    {}

  /* Temporary constructor */
  PileupData( double piledupCtRate,
	      double trueCtRate):
    _piledupCtRate(piledupCtRate),
    _trueCtRate(trueCtRate),
    _probSingleEvent(0.)
    {}

  double piledupCtRate() const
    {
      PROC;
      RETURN _piledupCtRate;
    }

  double trueCtRate() const
    {
      PROC;
      RETURN _trueCtRate;
    }

  double probSingleEvent() const
    {
      PROC;
      RETURN _probSingleEvent;
    }
  
private:

  PileupData();

  double _piledupCtRate;
  double _trueCtRate;
  double _probSingleEvent;
};

// ===========================================================================

// class PatternFuncs

// Contains pattern-dependent functions useful for pileup correction

// ===========================================================================

class PatternFuncs
{
public:
  
  /* Constructors */
  PatternFuncs(){};

  virtual ~PatternFuncs(){};

  double piledupCtRate( double energy, double flux) const
    {
      RETURN pileupData(energy, flux).piledupCtRate();
    };

  double trueCtRate( double energy, double flux)   const
    {
      RETURN pileupData(energy, flux).trueCtRate();
    };

  double probSingleEvent( double energy, double flux) const
    {
      RETURN pileupData(energy, flux).probSingleEvent();
    }
    
protected:

  virtual const PileupData pileupData ( double energy, double flux ) const = 0;

};
#endif
