// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: PiledUpResponseCorrector
// Author: Hassan Siddiqui
//
// Description: 
// Corrects response for pile-up
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: PiledUpResponseCorrector.h,v 1.1 2000/03/28 12:42:54 hsiddiqu Exp $

#ifndef PiledUpResponseCorrector_H
#define PiledUpResponseCorrector_H

// General classes

#include <iostream>
#include <vector>
#include <map>

// SAS-specific classes

#include "Dal.h"
#include "Tracer.h"

// Local Classes
#include "GenericResponse.h"
#include "Frame.h"
#include "ResponseCorrector.h"
#include "RmfUtils.h"

// -----------------------------------------------------------------------------------

// class PiledUpResponseCorrector

// Corrects response row for pile-up

// -----------------------------------------------------------------------------------

class PiledUpResponseCorrector : public ResponseCorrector
{

public:
  
  /* Constructor */

  PiledUpResponseCorrector( const GenericResponse  * response,
			    const Frame            * frame): 
    _response(response),
    _frame(frame)
    {
      PROC;
      ENDPROC;
     }

  ~PiledUpResponseCorrector()
    {
      PROC;
      ENDPROC;
    }

  /* Factory methods */

  const vector<double> correctedResponse( const Interval<double>& ebin0 ) const
    {
      PROC;

      /* Compute modified response due to either a single photon
	 of energy E0, or of
	 two photons of energies E1, E2, with a combined sum
	 corresponding to E0 impacting on CCD.

	 Expanded in terms of responses R, incident spectra S, and number of pixels
	 per CCD N, this is

	 R_pileup(E0) = R(E0) + 
	 sum(E1 = 0 to E0) [ S(E1) * S(E2) * [ R(E1) + R(E2) ] ] / 
	 ( S(E0) + sum(E1=0 to E0) [ S(E1) * S(E2) ] )

	 where E2 = E0 - E1.
      */

      /*
	 Note that pile up correction is ideally based on
	 incident spectrum. Using observed cts spectrum
	 as an approximation. Thus summing over channels,
	 not energy.
      */

      SAS_DEBUG( cout << "PileUpCorrector::correctedResponse called. " << endl;);

      vector<double> response0 = _response->responseVector(ebin0);

      double energy0 = 0.5 * ( ebin0.lo() + ebin0.hi() );

      vector<double> pileupsum;

      // Initialise pileupsum

      for (vector<double>::const_iterator i = response0.begin();
	   i != response0.end();
	   ++i)
	{

	  pileupsum.push_back(0.);
	}
      
      assert(pileupsum.size() == response0.size());

      Interval<double> ebin1;

      vector<double> responsesum;
      vector<double> pileupterm;
      double normalization = 0;

      SAS_DEBUG( cout << "ebinFirstIndex " <<_response->ebinFirstIndex() 
	     << "ebinLastIndex " <<_response->ebinLastIndex() 
	     << endl;);

      for (unsigned long row1 = _response->ebinFirstIndex(); 
	   row1 <= _response->ebinLastIndex(); 
	   ++row1)
	{

	      ebin1 = _response->ebins(row1);
	      
	      double energy1 = 0.5 * ( ebin1.lo() + ebin1.hi() );

	      if ( energy1 >= energy0 )
		{
		  /* 
		     Exit loop as we have 
		     now summed over all possible energies
		  */
		  
		  break;
		}
	      
	      double energy2 = energy0 - energy1;

	      SAS_DEBUG( cout << " energy0 = " << energy0
		     << " energy1 = " << energy1
		     << " energy2 = " << energy2
		     << endl;);

	      responsesum = RmfUtils::addResponse( _response->
						   responseVector(energy2),
						   _response->
						   responseVector(energy1) );

	      double jointProbability = probability(energy2) * probability(energy1);

	      pileupterm = RmfUtils::multiplyResponse( responsesum, 
						       jointProbability);

	      normalization +=jointProbability;

	      SAS_DEBUG( cout << "pileupterm sum " 
		     << RmfUtils::responseSum(pileupterm) << endl;);

 	      pileupsum = RmfUtils::addResponse (pileupsum, pileupterm);
	}
  
      SAS_DEBUG( cout << "pileupsum sum " 
	     << RmfUtils::responseSum(pileupsum) << endl;);

      pileupsum = RmfUtils::addResponse( pileupsum, response0);

//       normalization += probability(energy0);

//       cout << "Normalization = " << normalization;

//       assert (normalization > FLT64_MIN);
      
//       vector<double> piledupresponse = RmfUtils::multiplyReponse(pileupsum,
// 						       1/normalization);
      vector<double> piledupresponse = pileupsum;
      SAS_DEBUG(
      cout << "piledupresponse sum " 
	   << RmfUtils::responseSum(piledupresponse) << endl;
      
      cout << " Piled up response generated for energy " << energy0 
	   << " is as follows: " << endl;

      assert( piledupresponse.size() == response0.size() );

      int count = 0;

      vector<double>::const_iterator i_r = response0.begin();

      for ( vector<double>::const_iterator i_pr = piledupresponse.begin();
	    i_pr != piledupresponse.end();
	    ++i_pr)
	{
	  cout << " count " << count++ 
		 << " value " << *i_pr
		 << " difference " << (*i_pr) -(*i_r)
		 << endl;

	  ++i_r;
	}


      cout << " returning piledupresponse " << endl;
      )

      RETURN piledupresponse;
    }

private:

  /* Data members */
  const GenericResponse    *  _response;
  const Frame              *  _frame;

  /* Functions */

  /* Disable default constructor */
  PiledUpResponseCorrector();


  double probability ( double /* energy */ ) const
    {
      PROC;

//       double flux = 
// 	_frame->frameFlux(energy, 1
// 			  /*_instrumentData->coreData()->framePeriod()*/
// 			  );

      /* Determine probability (/sq. pixel)
	 of a photon of energy E hitting detector */

      
      /* Return a dummy value */
      RETURN 1;

    }
};

#endif






