// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: SquareResponseVector
// Author: Hassan Siddiqui
//
// Description: Produces a fragmented row response. 
// For testing the grouping algorithm in ResponseGrouping.h
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: SquareResponseVector.h,v 1.1 2000/03/28 12:43:00 hsiddiqu Exp $

#ifndef SquareResponseVector_H
#define SquareResponseVector_H

// General classes

#include <iostream>
#include "vector"

// SAS-specific classes

#include "Dal.h"

// Utility functions
#include "Tracer.h"

// Local functions 
#include "DummyResponseVector.h"

// -----------------------------------------------------------------------------------

// class SquareResponseVector

// Generates a simple square wave function, the throughs of which are less than the defined threshold.

// -----------------------------------------------------------------------------------

class SquareResponseVector : public DummyResponseVector
{

public:

  // The period of the square wave function is in terms of pha channels.
  SquareResponseVector(unsigned int vectorLength, 
		       double height = 1, 
		       unsigned int numberOfCycles = 5 ):
    _vectorLength(vectorLength),
    _height(height),
    _numberOfCycles(numberOfCycles)
    
    {
      PROC;
      ENDPROC;
    };

  const vector<double> response() const
    {
      PROC;
      
      unsigned int halfperiod = _vectorLength/(_numberOfCycles * 2);

      DEBUG (cout << " SquareResponseVector: halfperiod " 
	     << halfperiod << endl;);


      vector<double> response;

      for (unsigned int i = 0; i < _vectorLength; ++i)
	{
	      
// 	  // Increment periodicity as a function of position.

//  	  if (!(i%halfperiod) && (halfperiod > 10) ) halfperiod--;
	  if (halfperiod > 0)
	    {
	      if ((i/halfperiod)%2)
		{
		  response.push_back(_height);
		  
		  DEBUG (cout << " SquareResponseVector : " 
			 << i << " adding " << _height 
			 << " to list " << endl;) 
		    }
	      else
		{
		response.push_back(0);
		}
	    }
	  else
	    {
	      response.push_back(_height);
	    }

	}

      RETURN response;
    }



private:
  
  // Suppress default constructor
  SquareResponseVector();

  unsigned int _vectorLength;
  double _height;
  unsigned int _numberOfCycles;

};


#endif




