// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: VectorUtils
// Author: Hassan Siddiqui
//
// Description: 
// General Vector Utility functions
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: VectorUtils.h,v 1.2 1999/05/28 08:01:10 hsiddiqu Exp $

#ifndef VectorUtils_H
#define VectorUtils_H

// General classes

#include <iostream>
#include <vector>
#include <map>

// SAS-specific classes

#include "Dal.h"
#include "Tracer.h"

// Utility classes
#include "NumericalRange.h"

// Local Classes

// -----------------------------------------------------------------------------------

// class VectorUtils

// Suite of utility functions for vectors. 

// -----------------------------------------------------------------------------------

// class VectorUtils
// {

// public:


  template <class T> static T vectorSum(const vector<T>& vector0)
    {
      PROC;

      assert ( vector0.size() > 0);

      double sum = 0;
      for (vector<T>::const_iterator itr_vector = vector0.begin();
	   itr_vector != vector0.end();
	   ++itr_vector)
	{ 
	  sum += *itr_vector;
	}

      RETURN sum;
    };

  template <class T> vector<T> multiplyVector(const vector<T>& vector1, 
					      const vector<T>& vector2)
    {
      PROC;

      assert ( vector1.size() == vector2.size() );

      vector<T> productVector;

      // Clear out productVector object
      productVector.erase(productVector.begin(), productVector.end());

      vector<T>::const_iterator itr_vector2 = vector2.begin();

      for (vector<T>::const_iterator itr_vector1 = vector1.begin();
	   itr_vector1 != vector1.end();
	   ++itr_vector1)
	{ 
	  productVector.push_back( (*itr_vector1) * (*itr_vector2) );

	  ++itr_vector2;
	}

      assert ( vector1.size() == productVector.size() );

      RETURN productVector;
    };

  template <class T> static const vector<T> multiplyVector(const vector<T>& vector0, 
					       const double factor)
    {
      PROC;

      vector<T> productVector;

      // Clear out productVector object
      productVector.erase(productVector.begin(), productVector.end());

      for (vector<T>::const_iterator itr_vector = vector0.begin();
	   itr_vector != vector0.end();
	   ++itr_vector)
	{ 
	  productVector.push_back( (*itr_vector) * factor );
	}

      assert ( vector0.size() == productVector.size() );

      RETURN productVector;
    };


  template <class T> static const vector<T> addVector(const vector<T>& vector1, 
					  const vector<T>& vector2) 
    {
      PROC;

      assert ( vector1.size() == vector2.size() );

      vector<T> sumVector;

      // Clear out sumVector object
      sumVector.erase(sumVector.begin(), sumVector.end());

      vector<T>::const_iterator itr_vector2 = vector2.begin();

      for (vector<T>::const_iterator itr_vector1 = vector1.begin();
	   itr_vector1 != vector1.end();
	   ++itr_vector1)
	{ 
	  sumVector.push_back( (*itr_vector1) + (*itr_vector2) );

	  ++itr_vector2;
	}

      assert ( vector1.size() == sumVector.size() );

      RETURN sumVector;
    };

  template <class T> static const vector<T> normalizeVector(const vector<T>& vector0)
    {
      PROC;

      double normalization = vectorSum(vector0);

      // Make sure that it is not close to zero
      assert(normalization > FLT64_MIN);

      RETURN multiplyVector(vector0, (1/normalization));
    };

#endif






